@@ -19,26 +19,26 @@ clsmap = {
 try:
     import beaker.ext.memcached as memcached
     clsmap['ext:memcached'] = memcached.MemcachedNamespaceManager
-except InvalidCacheBackendError:
-    pass
+except InvalidCacheBackendError, e:
+    clsmap['ext:memcached'] = e
 
 try:
     import beaker.ext.database as database
     clsmap['ext:database'] = database.DatabaseNamespaceManager
-except InvalidCacheBackendError:
-    pass
+except InvalidCacheBackendError, e:
+    clsmap['ext:database'] = e
 
 try:
     import beaker.ext.sqla as sqla
     clsmap['ext:sqla'] = sqla.SqlaNamespaceManager
-except InvalidCacheBackendError:
-    pass
+except InvalidCacheBackendError, e:
+    clsmap['ext:sqla'] = e
 
 try:
     import beaker.ext.google as google
     clsmap['ext:google'] = google.GoogleNamespaceManager
-except (InvalidCacheBackendError, SyntaxError):
-    pass
+except (InvalidCacheBackendError, SyntaxError), e:
+    clsmap['ext:google'] = e
 
 
 class Cache(object):
@@ -47,6 +47,8 @@ class Cache(object):
     def __init__(self, namespace, type='memory', expiretime=None, starttime=None, **nsargs):
         try:
             cls = clsmap[type]
+            if isinstance(cls, InvalidCacheBackendError):
+                raise cls
         except KeyError:
             raise TypeError("Unknown cache implementation %r" % type)
             
@@ -88,8 +90,7 @@ class Cache(object):
         kwargs.update(kw)
         c = Cache(self.namespace.namespace, type=type, **kwargs)
         return c._get_value(key, expiretime=expiretime, createfunc=createfunc, starttime=starttime)
-    _legacy_get_value = util.deprecated(_legacy_get_value, "Specifying a 'type' and other namespace configuration with cache.get()/put()/etc. is depreacted.  Specify 'type' and other namespace configuration to cache_manager.get_cache() and/or the Cache constructor instead.")
-    
+    _legacy_get_value = util.deprecated(_legacy_get_value, "Specifying a 'type' and other namespace configuration with cache.get()/put()/etc. is deprecated. Specify 'type' and other namespace configuration to cache_manager.get_cache() and/or the Cache constructor instead.")
     def clear(self):
         self.namespace.remove()
     
